/* adsr.c
 * ------------------------------------------------------------------------
 *
 * Copyright (C) 2000 Stefan Fendt <stefan@lionfish.ping.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * ------------------------------------------------------------------------
 */

#include <math.h>
#include <stdio.h>

float adsr(int nr  , float A , float D , float S, float R,
	   float AV, float DV, int trigger)
{
  static int   timecount[16];
  static float Vol[16],
               lastVol[16],
               AQ[16],DQ[16],RQ[16],c[16],V[16];
  static int   AT[16],DT[16],ST[16],RT[16];

  if( trigger==1 )
    {
      timecount[nr]=0;

      AT[nr]=(A+1)*44100/1000;
      DT[nr]=(D+2)*44100/1000+AT[nr];
      ST[nr]=(S+3)*44100/1000+DT[nr];
      RT[nr]=(R+4)*44100/1000+ST[nr];

      AQ[nr]=2*3.141592/(float)AT[nr];
      DQ[nr]=2*3.141592/(float)(DT[nr]-AT[nr]);
      RQ[nr]=2*3.141592/(float)(RT[nr]-ST[nr]-500);

      if(AQ[nr]>1 || AQ[nr]<0)
	AQ[nr]=1;
      if(DQ[nr]>1 || DQ[nr]<0)
	DQ[nr]=1;
      if(RQ[nr]>1 || RQ[nr]<0)
	RQ[nr]=1;

      Vol[nr]=0;
      lastVol[nr]=0;
      c[nr]=1;
      V[nr]=0;
    }

  if( (timecount[nr]==0) )
    {
      V[nr]=AV;
      c[nr]=AQ[nr];
    }
  else
  if(timecount[nr]==AT[nr] )              /* Decay-Phase */
    {
      V[nr]=DV;
      c[nr]=DQ[nr];
    }
  else
  if(timecount[nr]==DT[nr] )            /* Sustain-Phase */
    {
      c[nr]=0;
    }
  else
  if(timecount[nr]==ST[nr] )          /* Release-Phase */
    {
      V[nr]=0;
      c[nr]=RQ[nr];
    }

  timecount[nr]++;
  Vol[nr]=V[nr]*c[nr]+lastVol[nr]*(1-c[nr]);
  lastVol[nr]=Vol[nr];

  return Vol[nr];
}







