/* main.c
 * ------------------------------------------------------------------------
 *
 * Copyright (C) 2000 Stefan Fendt <stefan@lionfish.ping.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * ------------------------------------------------------------------------
 */

#include <stdio.h>
#include <time.h>
#include <math.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include "parser.h"
#include "rc-filter.h"
#include "chorus.h"
#include "phaser.h"
#include "delay.h"
#include "adsr.h"
#include "osc.h"
#include "file_io.c"
#include "overdrive.h"

int main(int argc, char *argv[])
{
  FILE *infile;
  FILE *outfile;

  int length,trigger,i,j,l,nr;

  int output_is_dsp = 1;
  int pcm_bits = 16;
  int pcm_channels = 1;
  int pcm_rate = 44100;
  
  float time;
  float outvalue, outvalue_old;
  float sh1=0.5;
  float sh2=0.5;
  extern float chorus_level;
  extern float phaser_level;
  extern float master_vol;
  extern float osc_out[7];
  extern float osc_freq[7],    osc_wave[7];
  extern float osc_mul;
  extern float attack[12],     decay[12],      sustain[12],    release[12];
  extern float att_val[12],    dec_val[12];
  extern float flt_attack,     flt_decay,      flt_sustain,    flt_release;
  extern float res_attack,     res_decay,      res_sustain,    res_release;
  extern float flt_att_val,    flt_dec_val;
  extern float res_att_val,    res_dec_val;
  extern float sh_time,        sh_reg1,        sh_reg2;
  extern float hflt_attack,    hflt_decay,     hflt_sustain,   hflt_release;
  extern float hres_attack,    hres_decay,     hres_sustain,   hres_release;
  extern float hflt_att_val,   hflt_dec_val;
  extern float hres_att_val,   hres_dec_val;
  extern float drive;
  extern float delaytime,      delaylevel,     delayfeedback,  delayaddition;
  extern float vowel_time;
  extern char vowel_string[1024];

  float out[6];
  float last_out[7];
  float fm[6]={0,0,0,0,0,0};
  float phase[6]={0,0,0,0,0,0};
  float error_val=0;
  float max_val=0;
  float last_sh1,last_sh2,ssh1,ssh2;

  extern int modmatrix[6][7];
  extern int osc_trans[6];

  if(argc != 3)
    {
      fprintf(stderr,"\f\n\n");
      fprintf(stderr,"============================================================\n");
      fprintf(stderr,"==                                                        ==\n");
      fprintf(stderr,"==  B-A-C-T-E-R-I-U-M                                     ==\n");
      fprintf(stderr,"==  Analouge-Synthesizer-Simulation                       ==\n");
      fprintf(stderr,"==  (C) 1999,2000 Stefan Fendt                            ==\n");
      fprintf(stderr,"==                                                        ==\n");
      fprintf(stderr,"============================================================\n");
      fprintf(stderr,"\n");
      fprintf(stderr,"Version...: %s",VERSION);
      fprintf(stderr,"\n\nUSAGE: BacteriuM <parameter-file> <out-file> \n\n");
      fprintf(stderr,"\n");
      fprintf(stderr,"<parameter-file> this has to be one parameterfile as to\n");
      fprintf(stderr,"                 be found in the examples directory.\n\n");
      fprintf(stderr,"\n");
      fprintf(stderr,"<outfile>        this has to be the name of the wavefile\n");
      fprintf(stderr,"                 which should be generated.\n\n");
      fprintf(stderr,"\n");
      fprintf(stderr,"For a description of the parameters and settings have a \n");
      fprintf(stderr,"deeper look at 'TEMPLATE.synth'. This file contains all \n");
      fprintf(stderr,"possible parameter-file commands as well as short descrip-\n");
      fprintf(stderr,"tions of these.                                         \n");
      exit(1);
    } 

  if((outfile = fopen(argv[2],"wb"))==NULL)
    {
      fprintf(stderr,"\nERROR: can't write file %s !!\n",argv[1]);
      exit(1);
    }

  if((infile=fopen(argv[1],"rb"))==NULL)
    {
      fprintf(stderr,"\nERROR: can't open file %s !!\n",argv[2]);
      exit(1);
    }

  printf("\f\n\n");
  printf("============================================================\n");
  printf("==                                                        ==\n");
  printf("==  B-A-C-T-E-R-I-U-M                                     ==\n");
  printf("==  Analouge-Synthesizer-Simulation                       ==\n");
  printf("==  (C) 1999,2000 Stefan Fendt                            ==\n");
  printf("==                                                        ==\n");
  printf("============================================================\n");
  printf("\n");
  printf("Version...: %s (compiled on %s)\n",VERSION,__DATE__);
  printf("\n");
  printf("WARNING...: This software is protected by the GNU-general-\n");
  printf("            public-license. Do *not* use if you didn't read\n");
  printf("            and understood the file COPYING !");
  printf("\n\n");
   
  parse_par_file(infile);

  /*
   * determine length of the sound in Samples
   */

  length=0;
  for(i=0;i<6;i++) 
    if(length<(attack[i]+decay[i]+sustain[i]+release[i]))
       length=attack[i]+decay[i]+sustain[i]+release[i];

  length=length+delayaddition;

  length=length*44100/1000;
  
  printf("generated File will have %i Samples (%3.1f sec.) ...\n",
	 length,((float)length/44100));

  /*
   * calculate transposed osc-frequencies
   */

  for(i=0;i<6;i++) 
    if(osc_trans[i]==1) osc_freq[i]*=osc_mul;

  printf("\nStarting calculation\n\n");

  if ( ioctl(fileno(outfile), SOUND_PCM_WRITE_BITS, &pcm_bits) == -1) {
	printf("IOCTL SOUND_PCM_WRITE_BITS(%d) failed.\n", pcm_bits);
	output_is_dsp = 0;
  } else if ( ioctl(fileno(outfile),
			SOUND_PCM_WRITE_CHANNELS, &pcm_channels) == -1 ) {
	printf("IOCTL SOUND_PCM_WRITE_CHANNELS(%d) failed.\n", pcm_channels);
	output_is_dsp = 0;
  } else if ( ioctl(fileno(outfile), SOUND_PCM_WRITE_RATE, &pcm_rate) == -1 ) {
	printf("IOCTL SOUND_PCM_WRITE_RATE(%d) failed.\n", pcm_rate);
	output_is_dsp = 0;
  }

  if ( output_is_dsp ) {
	printf("All IOCTL calls accepted.\n");
	printf("Outfile is DSP device.\n");
  } else {
	printf("Outfile is WAV file.\n");
	init_wav_header(44100,1,length);
	write_wav_header(outfile);
  }
         
  time=(float)clock()/CLOCKS_PER_SEC;

  trigger=1;j=0;

  for(i=length+1;i!=0;i--)
    {
      if((i & 2047)==0)
	printf("\r[%3.2f%%] done ...",100-((float)i/(float)length)*100);
      
      for (nr=0;nr<6;nr++)
	{
	  last_out[nr]= osc_out[nr];
	  osc_out[nr] = osc(nr,              
			    osc_freq[nr]+
			    osc_freq[nr]*fm[nr]+
			    adsr ( nr+6,
				   attack[nr+6],
				   decay[nr+6],
				   sustain[nr+6],
				   release[nr+6],
				   att_val[nr+6],
				   dec_val[nr+6],
				   trigger),
			    osc_wave[nr],
			    0,phase[nr]) * 1;
	  
	  osc_out[nr] = osc_out[nr] * adsr ( nr,
					     attack[nr],
					     decay[nr],
					     sustain[nr],
					     release[nr],
					     att_val[nr],
					     dec_val[nr],
	  				     trigger);			     

	  out[nr]=osc_out[nr];
	}

      for (nr=0;nr<6;nr++)
	{
	  for(l=0;l<6;l++)
	    {
	      switch( modmatrix[nr][l] )
		{
		case 1:
		  out[nr] *= osc_out[l];
		  break;
		case 2:
		  if(osc_out[l]!=0)
		    {
		      if(osc_out[l]!=0) 
			out[nr] /= osc_out[l];
		      else
			out[nr] =0;
		    }
		  break;
		case 3:
		  out[nr] *= (osc_out[l]+1)/2;
		  break;
		case 4:
		  fm[nr] = osc_out[l];
		  break;
		case 5:
		  if(osc_out[l]>=0 && last_out[l]<0) out[nr]=osc(nr,0,0,1,0);
		  break;
		case 6:
		  phase[nr] = osc_out[l];
		  break;
		case 7:
		  if(osc_out[nr]>0.1)
		    out[nr] = out[nr];
		  else
		    out[nr] = out[l];		    
		  break;
		}
	    }
	}

      out[0]*=(float)modmatrix[0][6];
      out[1]*=(float)modmatrix[1][6];
      out[2]*=(float)modmatrix[2][6];
      out[3]*=(float)modmatrix[3][6];
      out[4]*=(float)modmatrix[4][6];
      out[5]*=(float)modmatrix[5][6];

      if(sh_time>0)
	{
	if(j>(sh_time*44100/1000))
	  {
	    j=0;
	    sh1=(osc_out[(int)sh_reg1]+1)/2+0.5;
	    sh2=(osc_out[(int)sh_reg2]+1)/2+0.5;
	    ssh1=sh1*0.6+0.01; 
	    ssh2=sh2*0.6+0.01; 
	  }
	j++;
	}
      else
	{sh1=sh2=last_sh1=last_sh2=ssh1=ssh2=1;}

      sh1=ssh1*0.01+last_sh1*0.99;
      sh2=ssh2*0.01+last_sh2*0.99;
      last_sh1=sh1;
      last_sh2=sh2;

      outvalue=(out[0]+out[1]+out[2]+
                out[3]+out[4]+out[5]);

      outvalue*=32767/6;

#if 1
      outvalue=rc_lowpass(adsr ( 12,
			         flt_attack,
                                 flt_decay,
                                 flt_sustain,
                                 flt_release,
	                         flt_att_val,
                                 flt_dec_val,
                                 trigger)*sh1,
			  adsr ( 13,
			         res_attack,
                                 res_decay,
                                 res_sustain,
                                 res_release,
	                         res_att_val,
                                 res_dec_val,
                                 trigger)*sh2,
			  outvalue);
            
      outvalue=rc_highpass(adsr ( 14,
			         hflt_attack,
                                 hflt_decay,
                                 hflt_sustain,
                                 hflt_release,
	                         hflt_att_val,
                                 hflt_dec_val,
                                 trigger)*sh1,
			  adsr ( 15,
			         hres_attack,
                                 hres_decay,
                                 hres_sustain,
                                 hres_release,
	                         hres_att_val,
                                 hres_dec_val,
                                 trigger)*sh2,
			  outvalue);
#endif

      if(vowel_time!=0) outvalue= vocal_formant(outvalue,vowel_string,vowel_time);
 
      if(drive>0) outvalue=overdrive(outvalue,drive);
      
      
      if(chorus_level>0) outvalue=chorus(outvalue,chorus_level); 

      if(phaser_level>0) outvalue=phaser(outvalue,phaser_level);
       
      if(delaytime>0) outvalue=delay(outvalue,delaytime,delaylevel,delayfeedback);
     
      outvalue=outvalue*master_vol;

      if(outvalue>max_val) max_val=outvalue;
      if(outvalue<(-1*max_val)) max_val=-1*outvalue;

      /* Here we calculate the noise-shaping. The factor of 1.2 in
	 the line below is necessary to prevent us from adding digital
         hissing. The improve in soundquality is very very little but
         audiable with good equipement.
      */

      if(outvalue>32765) outvalue=32765;
      if(outvalue<-32765) outvalue=-32765;
      outvalue=outvalue+error_val*1.2;     
      error_val=(outvalue+32768)-((int)(outvalue+32768)&0xffff);

      /* write the stuff to disk */

      putsample(outfile,(int)outvalue);
      trigger=0;
    }

  time=((float)clock()/CLOCKS_PER_SEC)-time;

  /* after this point we print out just some statistics...
   *
   */

  printf("\r[100%%] done ...\n");

  if( (max_val/32767*100)>99.9 && 
      (max_val/32767*100)<100.1 )
    printf("\nMaximum Amplitude was 100%%\n");
  else
    {
      printf("\nMaximum Amplitude was %3.1f%%.\n",max_val/32767*100);
      printf("Mastervolume could have been set to %1.3f without clipping.\n",32767/max_val*master_vol);
    }

  printf("Calculation used %3.1f seconds.",time);
  printf("This is %2.2f times faster(?) than realtime.\n",((float)length/44100)/time);

  return 0;
}












