/* osc.c
 * ------------------------------------------------------------------------
 *
 * Copyright (C) 2000 Stefan Fendt <stefan@lionfish.ping.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * ------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>
#include "osc.h"
#define SAMPLERATE 44100

float osc(const int nr,float frequenz, const int waveform, int sync, float pw)
{
  #define M_PI 3.1415926535897932384626433832795029

  static float wavetable[8][256][1024];
  static int wavetable_init=0;
  static float phase[12];
 
  float d0,d1,d2,d3,a,b,c;
  float value;
  float value2;
  float div;
  static float v0[12];
  static float v1[12];
  static float v2[12];
  static float v3[12];
  float delta;
  float k;

  /* Generate a bandlimited wavetable...
   * This is needed to avoid aliasing which doesn't make
   * a nice analog sound. 
   */
  if(wavetable_init==0)
  {
      float i,j;
      wavetable_init=1;

      fprintf(stderr,"*** generating bandlimited waveform-table ");

      /* Index 0 => Sinus */
      for(k=1;k<257;k++)
	  for(i=0;i<1024;i++)
	      wavetable[0][(int)k-1][(int)i]=sin(M_PI*2*i/1024.f);
      /* Index 1 => Triangle */
      for(k=1;k<257;k++)
	  for(i=0;i<1024;i++)
	  {
	      value=0;
	      for(j=1;j<50;j+=2)
	      {
		  if((25*k*j)<19000)
		      value+=1/(j*j)*cos(j*2*M_PI*i/1024.f);
	      }
	      wavetable[1][(int)k-1][(int)i]=value*4.888/6;
	  }
      /* Index 2 => Sawtooth */
      for(k=1;k<257;k++)
	  for(i=0;i<1024;i++)
	  {
	      value=0;
	      for(j=1;j<50;j++)
	      {
		  if((25*k*j)<19000)
		      value+=1/j*sin(j*2*M_PI*i/1024.f);
	      }
	      wavetable[2][(int)k-1][(int)i]=value*3.286/6;
	  }
      fprintf(stderr,"... done\n");
  }

  phase[nr] += frequenz/(float)SAMPLERATE*1024.f;
  if(sync==1) phase[nr]=0;
  k=frequenz/25;
  if(k>255) k=255;

  if(phase[nr]>=1023) phase[nr]-=1023.f;

  div = phase[nr]-(int)phase[nr];

  if(waveform!=4)
      if(waveform!=3)
      {
	  d0 = wavetable[waveform][(int)k][ (int)( phase[nr]+0 )&1023 ];
	  d1 = wavetable[waveform][(int)k][ (int)( phase[nr]+1 )&1023 ];
	  d2 = wavetable[waveform][(int)k][ (int)( phase[nr]+2 )&1023 ];
	  d3 = wavetable[waveform][(int)k][ (int)( phase[nr]-1 )&1023 ];
	  
	  a  = ( 3*(d0-d1) - d3 + d2 )/2;
	  b  = 2*d1 + d3 - ( 5*d0 + d2 )/2;
	  c  = (d1-d3)/2;
	  
	  value = a * div*div*div + b * div*div + c * div +d0;
      }
      else
      {
	  d0 = wavetable[2][(int)k][ (int)( phase[nr]+0 )&1023 ];
	  d1 = wavetable[2][(int)k][ (int)( phase[nr]+1 )&1023 ];
	  d2 = wavetable[2][(int)k][ (int)( phase[nr]+2 )&1023 ];
	  d3 = wavetable[2][(int)k][ (int)( phase[nr]-1 )&1023 ];
	  
	  a  = ( 3*(d0-d1) - d3 + d2 )/2;
	  b  = 2*d1 + d3 - ( 5*d0 + d2 )/2;
	  c  = (d1-d3)/2;
	  
	  value = a * div*div*div + b * div*div + c * div +d0;      
	  
	  d0 = wavetable[2][(int)k][ (int)( 512+pw*512.f-phase[nr]+0 )&1023 ];
	  d1 = wavetable[2][(int)k][ (int)( 512+pw*512.f-phase[nr]+1 )&1023 ];
	  d2 = wavetable[2][(int)k][ (int)( 512+pw*512.f-phase[nr]+2 )&1023 ];
	  d3 = wavetable[2][(int)k][ (int)( 512+pw*512.f-phase[nr]-1 )&1023 ];
	  
	  a  = ( 3*(d0-d1) - d3 + d2 )/2;
	  b  = 2*d1 + d3 - ( 5*d0 + d2 )/2;
	  c  = (d1-d3)/2;
	  
	  value2 = a * div*div*div + b * div*div + c * div +d0;      
	  
	  value=value+value2;
      }
  else
  {
      value=(float)(rand()&65535)/32768.f;
  }

  return value;
}








