/* parser.c
 * ------------------------------------------------------------------------
 *
 * Copyright (C) 2000 Stefan Fendt <stefan@lionfish.ping.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * ------------------------------------------------------------------------
 */

/*
 * suboptimal Parser for our .synth-files... 
 */

#include <math.h>
#include <stdlib.h>
#include "parser.h"
#include "token.h"

char outfile_name[160];

void parse_error(int i)
{
  printf("urgs... there's something wrong in line %i ...\n",i);
  exit(1);
}

void parse_par_file(FILE *infile)
{
   char line[161];
   char token[161];
   char parameter[161];
   char stringvalue[6][256];

   float value,value1,value2,value3,value4,value5,value6,value7;
   int i,j,k,l;

   printf("\nparsing parameter-file ...\n\n");

   i=0;

   while(NULL!=fgets(line,160,infile)) 
     {
       i++;

       if(line[0]!='#' && line[0]!=' ' && line[0]!='\n' )
	 {
	   if( sscanf(line,"%s %s",token,parameter) == 2 )
	     {
	       if(strcmp(token,"master_volume")==0)
		 {
		   if(sscanf(parameter,"%f",&value) != 1 ) parse_error(i);
		   master_vol=value;
		 }
	       if(strcmp(token,"overdrive_level")==0)
		 {
		   if(sscanf(parameter,"%f",&value) != 1 ) parse_error(i);
		   drive=value;
		   if(drive<=0.1) drive=0.1;
		 }
	       if(strcmp(token,"formantfilter")==0)
		 {
		     l=0;
		     while(parameter[l]!=',')
		     {
			 vowel_string[l]=parameter[l];
			 l++;
		     }
		     vowel_string[l]=0;

		     k=0;
		     l++;
		     while(parameter[l]!='\0')
		     {
			 stringvalue[0][k]=parameter[l];
			 l++;
			 k++;
		     }
		     stringvalue[0][k]=0;
		     vowel_time=atof(&stringvalue[0][0]);
		     printf("---> formantfilter is set to sequence %s and time %f.\n",
			    vowel_string,vowel_time);
		 }

	       if(strcmp(token,"transpose")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f,%f",
			     &value,
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6 ) != 7 ) parse_error(i);
		   
		   osc_mul=1;
		   
		   for(j=0;j<(int)value;j++)
		     {
		       osc_mul*=1.05946309436;
		     }
		   for(j=0;j>(int)value;j--)
		     {
		       osc_mul/=1.05946309436;
		     }
		   
		   osc_trans[0]=value1;
		   osc_trans[1]=value2;
		   osc_trans[2]=value3;
		   osc_trans[3]=value4;
		   osc_trans[4]=value5;
		   osc_trans[5]=value6;
		   
		 }
	       if(strcmp(token,"chorus")==0)
		 {
		   if(sscanf(parameter,"%f",&value) != 1 ) parse_error(i);
		   chorus_level=value;
		 }
	       if(strcmp(token,"phaser")==0)
		 {
		   if(sscanf(parameter,"%f",&value) != 1 ) parse_error(i);
		   phaser_level=value;
		 }
	       if(strcmp(token,"osc_waveforms")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6)!=6) parse_error(i);

		   osc_wave[0]=value1;
		   osc_wave[1]=value2;
		   osc_wave[2]=value3;
		   osc_wave[3]=value4;
		   osc_wave[4]=value5;
		   osc_wave[5]=value6;

		 }

	       if(strcmp(token,"osc_frequencies")==0)
		 {
		   k=0;
		   for(j=0;j<6;j++)
		     {
		       l=0;
		       while(parameter[k]!=',')
			 {
			   stringvalue[j][l++]=parameter[k++];
			 }
		       stringvalue[j][l]=0;
		       k++;

		       if ( (stringvalue[j][0]=='C' || stringvalue[j][0]=='c') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=261.63/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='C' || stringvalue[j][0]=='c') &&
			    (stringvalue[j][2]>='0' && stringvalue[j][2]<='9') &&
			    (stringvalue[j][1]=='#') )
			 osc_freq[j]=277.18/8*pow(2,(stringvalue[j][2]-'0'));
		       else
		       if ( (stringvalue[j][0]=='D' || stringvalue[j][0]=='d') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=293.67/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='D' || stringvalue[j][0]=='d') &&
			    (stringvalue[j][2]>='0' && stringvalue[j][2]<='9') &&
			    (stringvalue[j][1]=='#') )
			 osc_freq[j]=311.13/8*pow(2,(stringvalue[j][2]-'0'));
		       else
		       if ( (stringvalue[j][0]=='E' || stringvalue[j][0]=='e') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=329.63/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='F' || stringvalue[j][0]=='f') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=349.23/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='F' || stringvalue[j][0]=='f') &&
			    (stringvalue[j][2]>='0' && stringvalue[j][2]<='9') &&
			    (stringvalue[j][1]=='#') )
			 osc_freq[j]=369.99/8*pow(2,(stringvalue[j][2]-'0'));
		       else
		       if ( (stringvalue[j][0]=='G' || stringvalue[j][0]=='g') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=392.00/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='G' || stringvalue[j][0]=='g') &&
			    (stringvalue[j][2]>='0' && stringvalue[j][2]<='9') &&
			    (stringvalue[j][1]=='#') )
			 osc_freq[j]=415.30/8*pow(2,(stringvalue[j][2]-'0'));
		       else
		       if ( (stringvalue[j][0]=='A' || stringvalue[j][0]=='a') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=440.00/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='A' || stringvalue[j][0]=='a') &&
			    (stringvalue[j][2]>='0' && stringvalue[j][2]<='9') &&
			    (stringvalue[j][1]=='#') )
			 osc_freq[j]=466.16/8*pow(2,(stringvalue[j][2]-'0'));
		       else
		       if ( (stringvalue[j][0]=='H' || stringvalue[j][0]=='h') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=493.88/8*pow(2,(stringvalue[j][1]-'0'));
		       else
		       if ( (stringvalue[j][0]=='B' || stringvalue[j][0]=='b') &&
			    (stringvalue[j][1]>='0' && stringvalue[j][1]<='9') )
			 osc_freq[j]=493.88/8*pow(2,(stringvalue[j][1]-'0'));
		       else
			 osc_freq[j]=atof(&stringvalue[j][0]);
		       printf("---> OSC%1i set to %5.2f Hz (%s) \n",j,osc_freq[j],&stringvalue[j][0]);
		     }
		 }

	       if(strcmp(token,"lfo_waveforms")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6)!=6) parse_error(i);
		 }
	       if(strcmp(token,"lfo_is_fm")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6)!=6) parse_error(i);
		 }
	       if(strcmp(token,"osc_envelope")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6,
			     &value7)!=7) parse_error(i);

		   attack[(int)value1] =value2;
		   decay[(int)value1]  =value3;
		   sustain[(int)value1]=value4;
		   release[(int)value1]=value5;
		   att_val[(int)value1]=value6;
		   dec_val[(int)value1]=value7;
		   
		 }
	       if(strcmp(token,"frq_envelope")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6,
			     &value7)!=7) parse_error(i);

		   attack[6+(int)value1] =value2;
		   decay[6+(int)value1]  =value3;
		   sustain[6+(int)value1]=value4;
		   release[6+(int)value1]=value5;
		   att_val[6+(int)value1]=value6;
		   dec_val[6+(int)value1]=value7;
		   
		 }
	       if(strcmp(token,"filter_envelope")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4,
			     &value5,
			     &value6,
			     &value7)!=7) parse_error(i);

		   if(value1==0) /* setting lp-freq.-envelope */
		     {
		       flt_attack =value2;
		       flt_decay  =value3;
		       flt_sustain=value4;
		       flt_release=value5;
		       flt_att_val=2*3.1415*value6/44100/4;
		       flt_dec_val=2*3.1415*value7/44100/4;
		       if(flt_att_val>1) flt_att_val=1;
		       if(flt_dec_val>1) flt_dec_val=1;
		       if(flt_att_val<0) flt_att_val=0;
		       if(flt_dec_val<0) flt_dec_val=0;
		     }
		   else
		   if(value1==1) /* setting lp-reso.-envelope */
		     {
		       res_attack =value2;
		       res_decay  =value3;
		       res_sustain=value4;
		       res_release=value5;
		       res_att_val=value6/4;
		       res_dec_val=value7/4;
		     }
		   else
		   if(value1==2) /* setting hp-freq.-envelope */
		     {
		       hflt_attack =value2;
		       hflt_decay  =value3;
		       hflt_sustain=value4;
		       hflt_release=value5;
		       if(value6<10) value6=10;
		       if(value7<10) value7=10;
		       hflt_att_val=2*3.1415*value6/44100/4;
		       hflt_dec_val=2*3.1415*value7/44100/4;
		       if(hflt_att_val>1) hflt_att_val=1;
		       if(hflt_dec_val>1) hflt_dec_val=1;
		       if(hflt_att_val<0) hflt_att_val=0;
		       if(hflt_dec_val<0) hflt_dec_val=0;
		     }
		   else
		   if(value1==3) /* setting hp-reso.-envelope */
		     {
		       hres_attack =value2;
		       hres_decay  =value3;
		       hres_sustain=value4;
		       hres_release=value5;
		       hres_att_val=value6/4;
		       hres_dec_val=value7/4;
		     }

		 }
	       if(strcmp(token,"s&h")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f",
			     &value1,
			     &value2,
			     &value3)!=3) parse_error(i);

		   sh_time=value1;
		   sh_reg1=value2;
		   sh_reg2=value3;
		 }
	       if(strcmp(token,"delay")==0)
		 {
		   if(sscanf(parameter,"%f,%f,%f,%f",
			     &value1,
			     &value2,
			     &value3,
			     &value4)!=4) parse_error(i);
		   
		   delaytime=value1;
		   delaylevel=value2;
		   delayfeedback=value3;
		   delayaddition=value4;
		 }
	     }
	   else if( sscanf(line,"%s",token) == 1 )
	     {
	       if(strcmp(token,"modulation_matrix")==0)
		 {
		   for(j=0;j<6;j++)
		   {
		     fgets(line,160,infile);
		     i++;

		     if( sscanf(line,"%f,%f,%f,%f,%f,%f,%f",
				&value1,
				&value2,
				&value3,
				&value4,
				&value5,
				&value6,
				&value7)!=7) parse_error(i);

		     modmatrix[j][0]=(int)value1;
		     modmatrix[j][1]=(int)value2;
		     modmatrix[j][2]=(int)value3;
		     modmatrix[j][3]=(int)value4;
		     modmatrix[j][4]=(int)value5;
		     modmatrix[j][5]=(int)value6;
		     modmatrix[j][6]=(int)value7;
		   }
		 }
	     }
	   else
	     {
	       printf("%s",line);
	     }
	 }
     }
}
