/* waves.c
 * ------------------------------------------------------------------------
 *
 * Copyright (C) 2000 Stefan Fendt <stefan@lionfish.ping.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * ------------------------------------------------------------------------
 *
 * This file just calculates the waveforms for the synth. I think this to be
 * a lot smarter than carrying tons of wavedata in the distribution...
 * 
 * NOTE 28th of JAN: The white noise-section may create a "pumping"-effect
 *                   when replayed on frequencies higher than 1.0 Hz. If you 
 *                   wish to avoid these then use the according FM-LFO to 
 *                   heaviely modulate the output. This is just a work-arround.
 *                   The problem will be fixed in one of the next versions.
 *                   Another work-arround: use *very* high pitched noise-
 *                   frequencies 22 khz and above. 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void floatout(float wert)
{
  static int i,j;

  if(i==0)
    {
      printf("\n  {\n    ");
    }

  if(wert<0) printf("%1.6f",wert);
  else printf("%1.7f",wert);

  j++;
  i++;

  if(i<8192)
    if(j==7)
      {
	printf(",\n    ");
	j=0;
      }
    else
      {
	printf(",");
      }
  else
    {
      printf("\n  }\n");
      j=i=0;
    }
}

int main()
{
  int i,j;
  float wert,wert1,wert2,wert3,wert4,wert5,wert6,wert7,wert8,wert9;

  printf("float wavetable[9][8192] = \n{\n");

  for(i=0; i<8192; i++)
    {
      wert=sin((float)i*2*3.141592653/8192);
      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      wert=asin(sin((float)i*2*3.141592653/8192))/1.5707964;
      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      wert=asin(sin((float)i*3.141592653/8192+0.5*3.141592653))/1.5707964;
      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      wert=sin((float)i*2*3.141592653/8192);
      if(wert>0) wert=1; else wert=-1;

      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      j  = (int)((( (float) rand() / (RAND_MAX+1.0)*2)-1)*32767);
      j ^= (int)((( (float) rand() / (RAND_MAX+1.0)*2)-1)*32767);
      j ^= (int)((( (float) rand() / (RAND_MAX+1.0)*2)-1)*32767);
      j ^= (int)((( (float) rand() / (RAND_MAX+1.0)*2)-1)*32767);
      wert = (float)j/32767;
      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      wert=asin(sin((float)i*2*3.141592653/8193))/1.5707964;
      wert=(float)(((int)(wert*32767+32768)&0xfE00)-32768)/32767;
      wert*=wert*wert*wert*wert*wert;
      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      wert1=sin((float)i*1*2*3.141592653/8192);
      wert2=sin((float)i*2*2*3.141592653/8192)*0.5;
      wert3=sin((float)i*3*2*3.141592653/8192)*0.3;
      wert4=sin((float)i*4*2*3.141592653/8192)*0.2;
      wert5=sin((float)i*5*2*3.141592653/8192)*0.25;
      wert6=sin((float)i*6*2*3.141592653/8192)*0.25;
      wert7=sin((float)i*7*2*3.141592653/8192)*0.3;
      wert8=sin((float)i*8*2*3.141592653/8192)*0.2;
      wert9=sin((float)i*9*2*3.141592653/8192)*0.15;
      wert=(wert1+wert2+wert3+wert4+wert5+wert6+wert7+wert8+wert9)/9;
      floatout(wert);
    }

  printf("  ,");

  for(i=0; i<8192; i++)
    {
      wert1=asin(sin((float)i*3.141592653/8192+0.5*3.141592653))/1.5707964;
      wert2=asin(sin((float)i*2*3.141592653/8192+0.5*3.141592653))/1.5707964;
      wert3=asin(sin((float)i*4*3.141592653/8192+0.5*3.141592653))/1.5707964;
      wert4=sin((float)i*2*3.141592653/8192);
      if(wert4>0) wert4=1;
      wert=(wert1+wert2+wert3+wert4)/4;
      floatout(wert);
    }

  printf("};\n");

  return 0;
}











